/* -*- buffer-read-only: t -*-
   !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
   This file is built by opcode.pl from its data.
   Any changes made here will be lost!
 */

PERL_CALLCONV OP *Perl_do_kv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aassign(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_abs(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_accept(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_add(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aeach(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aelem(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aelemfast(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aelemfastlex_store(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_akeys(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_alarm(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_and(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_anoncode(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_anonconst(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_anonhash(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_anonlist(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_argcheck(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_argdefelem(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_argelem(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_aslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_atan2(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_av2arylen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_avhvswitch(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_backtick(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_bind(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_binmode(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_bit_and(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_bit_or(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_bless(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_blessed(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_break(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_caller(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_catch(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ceil(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_chdir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_chop(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_chown(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_chr(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_chroot(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_clonecv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_close(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_closedir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_cmpchain_and(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_cmpchain_dup(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_complement(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_concat(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_cond_expr(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_const(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_continue(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_coreargs(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_crypt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_dbmopen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_dbstate(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_defined(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_delete(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_die(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_divide(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_each(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ehostent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_emptyavhv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_enter(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_entereval(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_entergiven(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_enteriter(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_enterloop(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_entersub(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_entertry(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_entertrycatch(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_enterwhen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_enterwrite(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_eof(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_eq(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_exec(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_exists(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_exit(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_fc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_fileno(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_flip(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_flock(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_floor(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_flop(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_fork(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_formline(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ftis(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ftlink(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ftrowned(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ftrread(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_fttext(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_fttty(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ge(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gelem(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getlogin(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getpeername(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getpgrp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getppid(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_getpriority(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ggrent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ghostent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_glob(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gmtime(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gnetent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_goto(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gprotoent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gpwent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_grepstart(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_grepwhile(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gservent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_gvsv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_helem(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_hintseval(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_hslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_add(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_divide(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_eq(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_ge(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_gt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_le(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_lt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_modulo(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_multiply(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_ncmp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_ne(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_negate(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_i_subtract(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_index(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_int(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_introcv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ioctl(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_is_bool(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_is_tainted(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_is_weak(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_isa(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_iter(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_join(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_kvaslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_kvhslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_last(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_le(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leave(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leaveeval(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavegiven(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leaveloop(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavesub(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavesublv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavetry(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavetrycatch(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavewhen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_leavewrite(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_left_shift(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_length(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_link(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_list(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_listen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lock(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lvavref(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lvref(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_lvrefslice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_mapwhile(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_match(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_method(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_method_named(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_method_redir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_method_redir_super(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_method_super(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_mkdir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_modulo(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_multiconcat(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_multideref(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_multiply(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_nbit_and(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_nbit_or(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ncmp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ncomplement(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ne(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_negate(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_next(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_nextstate(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_not(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_null(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_oct(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_once(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_open(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_open_dir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_or(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ord(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pack(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padav(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padcv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padhv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padrange(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padsv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_padsv_store(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pipe_op(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_poptry(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pos(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_postdec(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_postinc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pow(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_predec(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_preinc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_print(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_prototype(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_prtf(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_push(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pushdefer(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_pushmark(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_qr(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_quotemeta(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rand(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_range(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rcatline(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_readdir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_readline(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_readlink(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_redo(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ref(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_refaddr(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_refassign(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_refgen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_reftype(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_regcomp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_regcreset(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rename(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_repeat(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_require(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_reset(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_return(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_reverse(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rewinddir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_right_shift(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rmdir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_runcv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rv2av(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rv2cv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rv2gv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_rv2sv(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sassign(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sbit_and(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sbit_or(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_schop(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_scmp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_scomplement(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_seekdir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_select(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_semctl(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_semget(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_seq(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_setpgrp(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_setpriority(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_shift(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_shmwrite(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_shostent(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_shutdown(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sin(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sle(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sleep(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_smartmatch(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sne(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_socket(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sockpair(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sort(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_splice(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_split(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sprintf(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_srand(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_srefgen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sselect(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ssockopt(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_stat(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_stringify(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_stub(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_study(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_subst(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_substcont(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_substr(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_subtract(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_syscall(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sysopen(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sysread(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_sysseek(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_system(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_syswrite(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_tell(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_telldir(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_tie(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_tied(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_time(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_tms(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_trans(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_truncate(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_uc(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_ucfirst(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_umask(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_undef(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_unpack(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_unshift(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_unstack(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_untie(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_unweaken(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_vec(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_wait(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_waitpid(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_wantarray(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_warn(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_weaken(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_pp_xor(pTHX) __attribute__visibility__("hidden");
PERL_CALLCONV OP *Perl_unimplemented_op(pTHX) __attribute__visibility__("hidden");

/* ex: set ro: */
